new;
library cml,qam,optmum,pgraph;

cls;

case = 1;

PathName = "data\\";

if case == 0;
  datFileName = PathName $+ "data-datastream";
else;
  datFileName = PathName $+ "data-bloomberg";
endif;
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

data = Fill_Miss(data,4);
if case /= 1;
  data = trimr(data,1,0);
endif;

Data = selif(Data,(Data[.,1] .>= 19991231) .and (Data[.,1] .<= 20091231));

Dates = Data[.,1];
nDates = rows(Dates);

SX5T = 100 * Data[.,3] ./ Data[1,3];
Eonia = Data[.,7];

Eonia_Index = Compute_Capitalized_Eonia(Dates,Eonia);
Eonia_Plus  = Compute_Capitalized_Eonia(Dates,Eonia + 30/1e4);

R_SX5T = SX5T ./ lag1(SX5T) - 1.0;
R_1M = lagn(SX5T,-22) ./ SX5T - 1.0;
R_Eonia = Eonia_Index ./ lag1(Eonia_Index) - 1.0;
R_Eonia_Plus = Eonia_Plus ./ lag1(Eonia_Plus) - 1.0;

{theta,cl,cov,logl} = regIGarch1(packr(R_SX5T),0,0);

lambda = theta[2];
sigma = Compute_EWMA_Volatility(Dates,SX5T,lambda);

proc (1) = Compute_Exposure(vol,vol_target,min_lev,max_lev);
  local lev;

  lev = vol_target/vol;
  lev = _min_(_max_(lev,min_lev),max_lev);

  retp(lev);
endp;

w = Compute_Exposure(sigma,0.15,0.50,1.50);
R1 = w .* R_SX5T + (1 - w) .* R_Eonia;
R2 = w .* R_SX5T - _max_(w - 1,0) .* R_Eonia_Plus + _max_(1 - w,0) .* R_Eonia;

R1[1] = 0.0; R2[1] = 0.0;

S1 = 100 * cumprodc(1 + R1);
S2 = 100 * cumprodc(1 + R2);

S = SX5T~S1~S2;
Results = zeros(5,3);
i = 1;
do until i > 3;
  report = Compute_Global_Reporting(Dates,S[.,i],SX5T,Eonia_Index,0,0);
  Results[.,i] = 100*vread(report,"mu") |
                 100*vread(report,"sigma") |
                 vread(report,"SR") |
                 vread(report,"IR") |
                 100*vread(report,"mDD") ;
  i = i + 1;
endo;

print ftosa(Results,"%lf",2,2);

x = seqa(0,1,101);
y1 = regKernel(100*R_1M,100*sigma,1,0,0,x);
y2 = regKernel(100*R_1M,100*sigma,3,0,0,x);

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 8; _pltype = 6|3; _pcolor = 1|4;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;
  fonts("simplex simgrma");

  setwind(1);
    _pcolor = 1;
    title("\216SX5T");
    xtics(0,nDates,2*262,1);
    ytics(40,120,20,2);
    let xlab = "2000" "02" "04" "06" "08" "10";
    asclabel(xlab,0);
    xy(0,SX5T); 

  setwind(2);
    _pcolor = 2; _pcross = 1;
    title("\216\202s\201]t[ / R]t[(1M)");
    xtics(0,100,20,2);
    ytics(-9,3,3,2);
    asclabel(0,0);
    xy(x,y1); 

  setwind(3);
    title("\216w]t[");
    _pcolor = 3; _pcross = 0;
    xtics(0,nDates,2*262,1);
    let xlab = "2000" "02" "04" "06" "08" "10";
    asclabel(xlab,0);
    ytics(0,200,50,2);
    xy(0,100*w);

  setwind(4);
    _pcolor = 4;
    title("\216S]t[");
    ytics(60,160,20,2);
    xy(0,S2); 

  endwind;