new;
library qam,pgraph,optmum;

cls;

// Simulation of a discrete random variable

proc (1) = simulate_F(r,c,x,cdf);
  local n,u,rnd,i,y;

  n = rows(x);
  u = rndu(r,c);
  rnd = zeros(r,c);
  i = 1;
  do until i > r;
    y = n + 1 - sumc(u[i,.] .<= cdf);
    y = missrv(miss(y,n + 1), n);
    rnd[i,.] = x[y]';
    i = i + 1;
  endo;

  retp(rnd);
endp;

rndseed 456;

_fcmptol = 1e-5;

S0 = 100;
Maturity = 1;
dt = 1/260;
t = seqa(0,dt,Maturity*260+1);
nT = rows(t);
t[nT] = Maturity;

nS = 1000;
nR = 1;
sigma = 0.05;
r = 0.05;

V0 = 100;
G = 100;
m = 5;
x_m = seqa(0,1/100,1001);

// Gaussian case

p_simulated = 0;
gamma_ = {};
i = 1;
do until i > nR;
  Rt = rndn(nT,nS) * sigma;
  Rt[1,.] = 0 * Rt[1,.];
  St = S0 .* cumprodc(1 + Rt);

  {F,V,D,tau,Loss} = cppi_simulate(r,m,V0,G,Maturity,t,St);
  p_simulated = p_simulated + meanc(D);
  gamma_ = gamma_ | delif(Loss,dotfeq(Loss,0));
  i = i + 1;
endo;
gamma_ = meanc(packr(gamma_));
p_simulated = p_simulated / nR;
R_minus = - (1 + (1-m) * r * dt) / m;
pi_ = cdfn(R_minus/sigma);
p_theoretical = 1 - (1-pi_)^260;
print ftos(100*p_simulated,"p (simulated) = %lf",3,2);
print ftos(100*p_theoretical,"p (theoretical) = %lf",3,2);
print ftos(gamma_,"gap risk = %lf",3,2);
print "--------------------------------------------";

R_minus = - (1 + (1 - x_m) * r * dt) ./ x_m;
pi_ = cdfn(R_minus/sigma);
p1 = 1 - (1-pi_)^260;

// Student case

nu = 3;
p_simulated = 0;
gamma_ = {};
i = 1;
do until i > nR;
  Rt = cdftci(rndu(nT,nS),nu) * sigma;
  Rt[1,.] = 0 * Rt[1,.];
  St = S0 .* cumprodc(1 + Rt);

  {F,V,D,tau,Loss} = cppi_simulate(r,m,V0,G,Maturity,t,St);
  p_simulated = p_simulated + meanc(D);
  gamma_ = gamma_ | delif(Loss,dotfeq(Loss,0));
  i = i + 1;
endo;
gamma_ = meanc(packr(gamma_));
p_simulated = p_simulated / nR;
R_minus = - (1 + (1-m) * r * dt) / m;
pi_ = 1 - cdftc(R_minus/sigma,nu);
p_theoretical = 1 - (1-pi_)^260;
print ftos(100*p_simulated,"p (simulated) = %lf",3,2);
print ftos(100*p_theoretical,"p (theoretical) = %lf",3,2);
print ftos(gamma_,"gap risk = %lf",3,2);
print "--------------------------------------------";

R_minus = - (1 + (1 - x_m) * r * dt) ./ x_m;
pi_ = 1 - cdftc(R_minus/sigma,nu);
p2 = 1 - (1-pi_)^260;

nS = 1000;
nR = 10;

// F1

let x_F1   = -0.15 -0.10 -0.05 0.00 0.05 0.10 0.15;
let pdf_F1 =  0.05  0.10  0.20 0.30 0.20 0.10 0.05;
cdf_F1 = cumsumc(pdf_F1);
p_simulated = 0;
gamma_ = {};
i = 1;
do until i > nR;
  Rt = simulate_F(nT,nS,x_F1,cdf_F1);
  Rt[1,.] = 0 * Rt[1,.];
  St = S0 .* cumprodc(1 + Rt);

  {F,V,D,tau,Loss} = cppi_simulate(r,m,V0,G,Maturity,t,St);
  p_simulated = p_simulated + meanc(D);
  gamma_ = gamma_ | delif(Loss,dotfeq(Loss,0));
  i = i + 1;
endo;
gamma_ = meanc(packr(gamma_));
p_simulated = p_simulated / nR;
R_minus = - (1 + (1-m) * r * dt) / m;
pi_ = sumc(pdf_F1 .* (x_F1 .<= R_minus));
p_theoretical = 1 - (1-pi_)^260;
print ftos(100*p_simulated,"p (simulated) = %lf",3,2);
print ftos(100*p_theoretical,"p (theoretical) = %lf",3,2);
print ftos(gamma_,"gap risk = %lf",3,2);
print "--------------------------------------------";

R_minus = - (1 + (1 - x_m) * r * dt) ./ x_m;
pi_ = sumc(pdf_F1 .* (x_F1 .<= R_minus'));
p3 = 1 - (1-pi_)^260;

// F2

let x_F2   = -0.15 -0.10 -0.05 0.00 0.05 0.10 0.15;
let pdf_F2 =  0.10  0.15  0.15 0.15 0.15 0.15 0.15;
cdf_F2 = cumsumc(pdf_F2);
p_simulated = 0;
gamma_ = {};
i = 1;
do until i > nR;
  Rt = simulate_F(nT,nS,x_F2,cdf_F2);
  Rt[1,.] = 0 * Rt[1,.];
  St = S0 .* cumprodc(1 + Rt);

  {F,V,D,tau,Loss} = cppi_simulate(r,m,V0,G,Maturity,t,St);
  p_simulated = p_simulated + meanc(D);
  gamma_ = gamma_ | delif(Loss,dotfeq(Loss,0));
  i = i + 1;
endo;
gamma_ = meanc(packr(gamma_));
p_simulated = p_simulated / nR;
R_minus = - (1 + (1-m) * r * dt) / m;
pi_ = sumc(pdf_F2 .* (x_F2 .<= R_minus));
p_theoretical = 1 - (1-pi_)^260;
print ftos(100*p_simulated,"p (simulated) = %lf",3,2);
print ftos(100*p_theoretical,"p (theoretical) = %lf",3,2);
print ftos(gamma_,"gap risk = %lf",3,2);
print "--------------------------------------------";

R_minus = - (1 + (1 - x_m) * r * dt) ./ x_m;
pi_ = sumc(pdf_F1 .* (x_F1 .<= R_minus'));
p4 = 1 - (1-pi_)^260;

all_p = p1~p2~p3~p4;
all_p = all_p .* (x_m .> 1);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3; _plwidth = 10; _pcolor = 1|2|3|4;
  xtics(0,10,1,2);
  _plegstr = "Normale\000Student\000F]1[\000F]2[";
  _plegctl = {2 5 1 5};
  xlabel("\216m");
  graphprt("-c=1 -cf=ex25-1.eps");
  xy(x_m,100*all_p);

mu = 0.15; 
r = 0.05;
T = 1;
C0 = S0 .* (1 - exp(-r .* T));

m = seqa(0,0.01,1001);
sigma = 0.10~0.20~0.30~0.75;

c = cdfn( (mu./sigma + (m - 3/2) .* sigma) .* sqrt(T) );
rho1 = c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);

r = 0.10;
c = cdfn( (mu./sigma + (m - 3/2) .* sigma) .* sqrt(T) );
rho2 = c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);

r = 0.05;
mu = -0.10; 
C0 = S0 .* (1 - exp(-r .* T));
c = cdfn( (mu./sigma + (m - 3/2) .* sigma) .* sqrt(T) );
rho3 = c .*  m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);

graphset;
  window3u;
    fonts("simplex simgrma");
    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 6|3; _plwidth = 10; _pcolor = 1|2|3|4;
    _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.275;
    xtics(0,10,1,2);
    xlabel("\216m");

  setwind(1);
    title("\216\202m\201 = 15% - r = 5%");
    ytics(0,100,20,2);
    _plegstr = "\202s\201 = 10%\000\202s\201 = 20%\000\202s\201 = 30%\000\202s\201 = 60%";
    _plegctl = {2 5 9 3};
    xy(m,100*rho1);

  _plegctl = 0;

  setwind(2);
    title("\216\202m\201 = 15% - r = 10%");
    ytics(0,80,20,2);
    xy(m,100*rho2);

  setwind(3);
    title("\216\202m\201 = -10% - r = 5%");
    ytics(0,10,2,2); 
    xy(m,100*rho3);

    graphprt("-c=1 -cf=ex25-2.eps");

  endwind;

mu = 0.15; 
r = 0.05;
dt = 1/260;
T = 1~2;
C0 = S0 .* (1 - exp(-r .* T));
m = seqa(0,0.1,251);
R_minus = - (1 + (1 - m) * r * dt) ./ m;

sigma = 0.20;
pi_ = cdfn(R_minus/(sigma*sqrt(dt)));
p = (1-pi_)^(T/dt);
c = cdfn( (mu./sigma + (m - 3/2) .* sigma) .* sqrt(T) );
rho = c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);
rho_gap_risk = p .* c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);
rho1 = rho[.,1]~rho_gap_risk[.,1]~rho[.,2]~rho_gap_risk[.,2];

sigma = 0.30;
pi_ = cdfn(R_minus/(sigma*sqrt(dt)));
p = (1-pi_)^(T/dt);
c = cdfn( (mu./sigma + (m - 3/2) .* sigma) .* sqrt(T) );
rho = c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);
rho_gap_risk = p .* c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);
rho2 = rho[.,1]~rho_gap_risk[.,1]~rho[.,2]~rho_gap_risk[.,2];

sigma = 0.50;
pi_ = cdfn(R_minus/(sigma*sqrt(dt)));
p = (1-pi_)^(T/dt);
c = cdfn( (mu./sigma + (m - 3/2) .* sigma) .* sqrt(T) );
rho = c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);
rho_gap_risk = p .* c .* m .* C0 ./ S0 .* exp( (m - 1) .* (mu - r - sigma^2) .* T);
rho3 = rho[.,1]~rho_gap_risk[.,1]~rho[.,2]~rho_gap_risk[.,2];

graphset;
  window3u;
    fonts("simplex simgrma");
    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 6|3; _plwidth = 10; _pcolor = 1|2|3|4;
    _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.275;
    xtics(0,25,5,2);
    xlabel("\216m");

  setwind(1);
    title("\216\202s\201 = 20%");
    _plegstr = "T = 1\000T = 1 (avec gap risk)\000T = 2\000T = 2 (avec gap risk)";
    _plegctl = {2 5 9 3};
    ytics(0,4000,500,2);
    xy(m,100*rho1);

  _plegctl = 0;

  setwind(2);
    title("\216\202s\201 = 30%");
    ytics(0,300,50,2);
    xy(m,100*rho2);

  setwind(3);
    title("\216\202s\201 = 50%");
    ytics(0,16,2,2);
    xy(m,100*rho3);

    graphprt("-c=1 -cf=ex25-3.eps");

  endwind;


/*
p (simulated) = 0.77
p (theoretical) = 0.83
gap risk = -0.10
--------------------------------------------
p (simulated) = 95.09
p (theoretical) = 97.46
gap risk = -2.34
--------------------------------------------
p (simulated) = 0.00
p (theoretical) = 0.00
gap risk =   .
--------------------------------------------
p (simulated) = 0.06
p (theoretical) = 0.00
gap risk =   .
--------------------------------------------
*/