new;
library qam,optmum,pgraph;

cls;

pqgwin many;

use_continuous = 1;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

Bounds = (-5~5) .* ones(n,1);

print "---------------------------";
print "---- Covariance matrix ----";
print ftosa(100*cov,"%lf",3,3);

let phi = -1.0 -0.5 -0.25 0.00 0.25 0.50 1.00 2.00;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,phi,0);  

Results = phi'|100*(x|er'|risk');

TexFile = "ex29.tex";
mask = 1;
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,1);

print "=====================  Question 1  =======================================";
print "--- Efficient Frontier ----";
print ftosa(Results,"%lf",3,3);

phi = seqa(-2,1/100,600);
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,phi,0);  
er0 = er; risk0 = risk;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xtics(10,40,5,2);
  ytics(0,12,2,0);
  title("\216Question 29.1");
  xlabel("\216Volatilite (en %)");
  ylabel("\216Rendement espere (en %)");
  _pmsgstr = "-\000-\000-";
  _pmsgctl = 4.8~0.31~0.07~45~2~0~5 |
             0.18~3.88~0.07~-45~2~0~5 |
             0.18~4.12~0.07~-45~2~0~5 ;
  graphprt("-c=1 -cf=ex29-1.eps");
  xy(100*risk0,100*er0);

print "=====================  Question 2  =======================================";

phi = 0;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,phi,0);  
x_min = x;
er_min = er;
risk_min = risk;  
print "Portefeuille de Variance Minimale";
print ftosa(100*x_min',"%lf",3,2); 
print ftos(100*risk_min,"vol = %lf",3,2); 
print ftos(100*er,"er = %lf",3,2);

Results = 100*(x|er|risk)|phi;

print "=====================  Question 3  =======================================";

vol_target = 0.10;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,vol_target,2);  
print ftosa(100*x',"%lf",3,2); 
print ftos(100*risk,"vol = %lf",3,2); 
print ftos(100*er,"er = %lf",3,2); 
print "------------------------";

Results = Results ~ (100*(x|er|risk)|phi);

vol_target = 0.15;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,vol_target,2);  
print ftosa(100*x',"%lf",3,2); 
print ftos(100*risk,"vol = %lf",3,2); 
print ftos(100*er,"er = %lf",3,2); 
print "------------------------";

Results = Results ~ (100*(x|er|risk)|phi);

vol_target = 0.20;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,vol_target,2);  
x_sharpe = x;
er_sharpe = er;
risk_sharpe = risk;   
print ftosa(100*x',"%lf",3,2); 
print ftos(100*risk,"vol = %lf",3,2);
print ftos(100*er,"er = %lf",3,2); 

Results = Results ~ (100*(x|er|risk)|phi);

fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);

print "=====================  Question 4  =======================================";

let alpha = -0.50 -0.25 0.00 0.10 0.20 0.50 0.70 1.00;
x = (1-alpha') .* x_min + alpha' .* x_sharpe;
er = x'mu;
risk = sqrt(diag(x'cov*x));
print "           ALPHA              VOL               ER";
print ftosa(100*(alpha~risk~er),"%lf",3,2); 

alpha1 = alpha;
er1 = x'mu;
risk1 = sqrt(diag(x'cov*x));

alpha = seqa(-1.0,5/1000,1000);
x = (1-alpha') .* x_min + alpha' .* x_sharpe;

er2 = x'mu;
risk2 = sqrt(diag(x'cov*x));

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 4;
  xtics(10,40,5,2);
  ytics(0,12,2,0);
  title("\216Question 29.4");
  xlabel("\216Volatilite (en %)");
  ylabel("\216Rendement espere (en %)");
  _pmsgstr = "-\000-\000-\000" $+ MakeMsgStr("\202a\201 = " $+ ftosa(alpha1,"%lf",3,2)) $+ "\000w]1[\000w]2[";
  _pmsgctl = 4.8~0.31~0.07~45~2~0~5 |
             0.18~3.88~0.07~-45~2~0~5 |
             0.18~4.12~0.07~-45~2~0~5 |
             100*risk1+0.5~100*er1-0.10~ones(rows(risk1),1) .* (0.10~0~1~2~6) |
             100*(risk_min|risk_sharpe)-1.75~100*(er_min|er_sharpe)-0.0~ones(2,1) .* (0.175~0~1~5~6) ;
  _psym = 100*risk1~100*er1~ones(rows(risk1),1) .* (8~2.5~2~1~10) |
          100*(risk_min|risk_sharpe)~100*(er_min|er_sharpe)~ones(2,1) .* (11~4~5~1~10) ;
  graphprt("-c=1 -cf=ex29-2.eps");
  xy(100*risk2,100*er2);

print "=====================  Question 5  =======================================";

Bounds = (0~1.00) .* ones(n,1);
phi = 0;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,phi,0);  
x_min = x;
vol_min = risk;

print ftosa(100*x_min',"%lf",3,2); 
print ftos(100*vol_min,"vol = %lf",3,2);
print ftos(100*er,"er = %lf",3,2);  
print "------------------------";

Results = 100*(x|er|risk)|phi;

vol_target = 0.10;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,vol_target,2);  
print ftosa(100*x',"%lf",3,2); 
print ftos(100*risk,"vol = %lf",3,2); 
print ftos(100*er,"er = %lf",3,2); 
print "------------------------";

Results = Results ~ (100*(x|er|risk)|phi);

vol_target = 0.15;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,vol_target,2);  
print ftosa(100*x',"%lf",3,2); 
print ftos(100*risk,"vol = %lf",3,2); 
print ftos(100*er,"er = %lf",3,2); 
print "------------------------";

Results = Results ~ (100*(x|er|risk)|phi);

vol_target = 0.20;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,vol_target,2);  
print ftosa(100*x',"%lf",3,2); 
print ftos(100*risk,"vol = %lf",3,2);
print ftos(100*er,"er = %lf",3,2);  

Results = Results ~ (100*(x|er|risk)|phi);

fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);

print "=====================  Question 6  =======================================";

mu = mu | 0.03;
cov = (cov ~ zeros(4,1)) | zeros(1,5);
Bounds = (-5~5) .* ones(n+1,1);
A = ones(1,n+1); B = 1;

phi = seqa(0,1/100,600);
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,phi,0);  
er3 = er; risk3 = risk;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3; _plwidth = 10; _pcolor = 3|1;
  xtics(0,40,5,2);
  ytics(0,12,2,0);
  title("\216Question 29.6(b)");
  xlabel("\216Volatilite (en %)");
  ylabel("\216Rendement espere (en %)");
  _pmsgstr = "-\000-\000-";
  _pmsgctl = 4.8~0.31~0.07~45~2~0~5 |
             0.18~3.88~0.07~-45~2~0~5 |
             0.18~4.12~0.07~-45~2~0~5 ;
  graphprt("-c=1 -cf=ex29-3.eps");
  xy(100*(risk3~risk0),100*(er3~er0));


indx = minindc(abs(x[5,.]'));
x_M = x[.,indx];

let phi = 0.25 0.50;
{x,er,risk,phi,retcode} = Qprog_Allocation_Solve(mu,cov,A,B,0,0,Bounds,phi,0);  

Results = phi'|100*(x|er'|risk');
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);

print "=====================  Question 6 (d)  =======================================";
print "Portefeuille w1";
print ftosa(100*x[.,1]',"%lf",3,2); 
print ftos(100*risk[1],"vol = %lf",3,2);
print ftos(100*er[1],"er = %lf",3,2);  
print "------------------------";
print "Portefeuille w2";
print ftosa(100*x[.,2]',"%lf",3,2); 
print ftos(100*risk[2],"vol = %lf",3,2);
print ftos(100*er[2],"er = %lf",3,2);  
print "------------------------";
sh = (er[2] - er[1])/(risk[2] - risk[1]);
print ftos(sh,"sh = %lf",3,2);  
print "------------------------";

x1 = x[.,1];
x2 = x[.,2];

alpha = x1[5]/(x1[5] - x2[5]);
x = (1-alpha) * x1 + alpha * x2;

print "=====================  Question 6 (e)  =======================================";
print ftos(alpha,"alpha = %lf",3,2);  
print ftosa(100*(x~x_M),"%lf",3,2); 
