new;
library pgraph,optmum,qam;

declare matrix _RFD_Sigma;
declare matrix _RFD_A;
declare matrix _RFD_Omega;
declare matrix _RFD_D;
declare matrix _RFD_P;
declare matrix _RFD_mtd;
declare matrix _RFD_retcode;

/*
**> Compute_Risk_Factor_Contribution
**
*/

proc (5) = Compute_Risk_Factor_Contribution(x,Sigma,A,Omega,D,mtd);
  local vol,y,mr,tr,rc;

  if mtd == 1;

    Sigma = A * Omega * A';
    vol = sqrt(x'Sigma*x);
    mr = Sigma*x/vol;

    mr = A' * mr;
    y = pinv(A) * x;
    tr = y .* mr;
    rc = tr/vol;

  elseif mtd == 2;

    Sigma = A * Omega * A' + D;
    vol = sqrt(x'Sigma*x);
    mr = Sigma*x/vol;

    mr = A' * mr;
    y = pinv(A) * x;
    tr = y .* mr;
    rc = tr/vol;

  else;
 
    vol = sqrt(x'Sigma*x);
    mr = Sigma*x/vol;
    tr = x .* mr;
    rc = tr/vol;
    y = x;

  endif;

  retp(vol,y,mr,tr,rc);
endp;

/*
**> Compute_RFD_Portfolio
**
*/

proc (6) = Compute_RFD_Portfolio(x0,Sigma,A,Omega,D,mtd);
  local n,x,fmin,grd,retcode;
  local vol,f,mr,tr,rc;
  local oldtrap;

  n = maxc(rows(Sigma)|rows(A));

  if x0 == 0;
    x0 = ones(n,1)/n;
  endif;
  
  if mtd == 1;
    _RFD_mtd = 1;
  elseif mtd == 2;
    _RFD_mtd = 2;
  else;
    _RFD_mtd = 0;
  endif;

  _RFD_Sigma = Sigma;
  _RFD_A = A;
  _RFD_Omega = Omega;
  _RFD_D = D;

  oldtrap = trapchk(1);
  trap 1,1;
  _RFD_P = pinv(A);
  trap oldtrap,1;
  if scalerr(_RFD_P) or (maxc(maxc(_RFD_P)) == __INFp);
    errorlog "error: can't compute the Moore-Penrose inverse of A.";
    _RFD_retcode = -1;
    retp(error(0),error(0),error(0),error(0),error(0),error(0));
  endif;

  _sqp_A = ones(1,n); _sqp_B = 1;
  _sqp_Bounds = 0~1;
  {x,fmin,grd,retcode} = sqpsolve(&_RFD_Portfolio,x0);

  _RFD_retcode = retcode;

  {vol,f,mr,tr,rc} = Compute_Risk_Factor_Contribution(x,Sigma,A,Omega,D,mtd);

  retp(x,vol,f,mr,tr,rc);
endp;

proc (1) = __RFD_Portfolio(theta);
  local x;
  x = sqrt(theta^2);
  x = x / sumc(x);
  retp(x);
endp;

proc (1) = _RFD_Portfolio(theta);
  local x,Sigma,mr,dev,vol,tr;
  local A,Omega,D,P,y;

  x = __RFD_Portfolio(theta);

  A = _RFD_A;
  Omega = _RFD_Omega;
  D = _RFD_D;
  P = _RFD_P;
  Sigma = _RFD_Sigma;

  if _RFD_mtd == 1;

    Sigma = A * Omega * A';
    vol = sqrt(x'Sigma*x);
    mr = Sigma*x/vol;
    mr = A'mr;
    y = P * x;
    tr = y .* mr;

  elseif _RFD_mtd == 2;

    Sigma = A * Omega * A' + D;
    vol = sqrt(x'Sigma*x);
    mr = Sigma*x/vol;
    mr = A'mr;
    y = P * x;
    tr = y .* mr;

  else;

    vol = sqrt(x'Sigma*x);
    mr = Sigma*x/vol;
    tr = x .* mr;

  endif;

  dev = sumc(vec(tr - tr')^2);

  retp(dev);
endp;

cls;

__output = 0;
_sqp_PrintIters = 0;
_sqp_MaxIters = 1000;

// Question 4

Let A[5,2] = 1.0 1.0
             1.0 1.0
             1.0 0.0 
             1.0 0.0
             1.0 0.0;
n = rows(A);
m = cols(A);

let sigma_F = 0.20 0.10;
let rho_F[2,2] = 1 0
                 0 1;
Omega = rho_F' .* sigma_F .* sigma_F';
let sigma_u = 0.10 0.15 0.10 0.15 0.20;
D = diagrv(eye(n),sigma_u^2);

cov = A*Omega*A' + D;
sigma = sqrt(diag(cov));
rho = cov ./ sigma ./ sigma';
Psi = A*Omega*A';

print "------------ Question 4.b ------------";
print ftosa(100*cov,"%lf",3,2);
print ftosa(100*sigma,"%lf",3,2);
print ftosa(100*rho,"%lf",3,2);
print " ";

all_x = eye(5)~(ones(5,1)/5);
Results = zeros(6,4);

i = 1;
do until i > 6;
  x = all_x[.,i];
  var_f = x'A*Omega*A'*x;
  var_s = x'D*x;
  var = var_f + var_s;
  p = var_f ./ var;
  Results[i,.] = var~var_f~var_s~p;
  i = i + 1;
endo;

print "------------ Question 4.c et 4.d ------------";
print ftosa(100*Results,"%lf",3,2);
print " ";

// Question 5

Let A[5,3] = 1.0 1.0 0.0
             1.0 1.0 0.0
             0.0 1.0 1.0 
             0.0 1.0 1.0
             0.0 1.0 1.0;
n = rows(A);
m = cols(A);

let sigma_F = 0.20 0.10;
sigma_F3 = sqrt(sigma_F[1]^2+sigma_F[2]^2);
sigma_F = sigma_F | sigma_F3;
let rho_F[2,2] = 1 0
                 0 1;
rho_13 = sigma_F[1]/sigma_F3;
rho_23 = -sigma_F[2]/sigma_F3;
rho_F = 1 | 0 | 1 | rho_13 | rho_23 | 1;
rho_F = xpnd(rho_F);

Omega = rho_F' .* sigma_F .* sigma_F';
let sigma_u = 0.10 0.15 0.10 0.15 0.20;
D = diagrv(eye(n),sigma_u^2);

cov = A*Omega*A' + D;
sigma = sqrt(diag(cov));
rho = cov ./ sigma ./ sigma';

print "------------ Question 5 ------------";
print ftos(100*sigma_F3,"sigma_F3 = %lf",3,2);
print ftos(100*rho_13,"rho_13 = %lf",3,2);
print ftos(100*rho_23,"rho_23 = %lf",3,2);
print ftosa(100*cov,"%lf",3,2);
print ftosa(100*sigma,"%lf",3,2);
print ftosa(100*rho,"%lf",3,2);
print " ";

// Question 6

Let A[5,2] = 1.0 1.0
             1.0 1.0
             1.0 0.0 
             1.0 0.0
             1.0 0.0;
n = rows(A);
m = cols(A);

let sigma_F = 0.20 0.10;
let rho_F[2,2] = 1 0
                 0 1;
Omega = rho_F' .* sigma_F .* sigma_F';
let sigma_u = 0.10 0.15 0.10 0.15 0.20;
D = diagrv(eye(n),sigma_u^2);

cov = A*Omega*A' + D;

x = ones(5,1)/5;
{vol,mr,tr,rc} = Compute_Risk_Contribution(x,cov);
results = 100*(x~mr~tr~rc);
print "------------ Question 6.f ------------";
print ftosa(results,"%lf",3,2);

{vol,y,mr,tr,rc} = Compute_Risk_Factor_Contribution(x,cov,A,Omega,D,1);
results = 100*(y~mr~tr~rc);
print ftosa(results,"%lf",3,2);

Let A[5,3] = 1.0 1.0 0.0
             1.0 1.0 0.0
             0.0 1.0 1.0 
             0.0 1.0 1.0
             0.0 1.0 1.0;
n = rows(A);
m = cols(A);

let sigma_F = 0.20 0.10;
sigma_F3 = sqrt(sigma_F[1]^2+sigma_F[2]^2);
sigma_F = sigma_F | sigma_F3;
let rho_F[2,2] = 1 0
                 0 1;
rho_13 = sigma_F[1]/sigma_F3;
rho_23 = -sigma_F[2]/sigma_F3;
rho_F = 1 | 0 | 1 | rho_13 | rho_23 | 1;
rho_F = xpnd(rho_F);

Omega = rho_F' .* sigma_F .* sigma_F';
let sigma_u = 0.10 0.15 0.10 0.15 0.20;
D = diagrv(eye(n),sigma_u^2);

cov = A*Omega*A' + D;

{vol,y,mr,tr,rc} = Compute_Risk_Factor_Contribution(x,cov,A,Omega,D,1);
results = 100*(y~mr~tr~rc);
print ftosa(results,"%lf",3,2);
print " ";

// Question 6.h

let sigma = 0.10 0.20 0.30 0.40;
let rho =   1.00
            0.90 1.00
            0.00 0.00 1.00
            0.00 0.00 0.40 1.00;
 
rho = xpnd(rho);
cov = sigma .* sigma' .* rho;
x0 = ones(4,1)/4;

{x,vol,mr,tr,rc} = Compute_ERC_Portfolio(x0,cov);
results = 100*(x~mr~tr~rc);
print "------------ Question 6.h ------------";
print ftosa(100*vol,"vol = %lf",3,2);
print ftosa(results,"%lf",3,2);

{x,vol,y,mr,tr,rc} = Compute_RFD_Portfolio(0,cov,0,0,0,0);

results = 100*(y~mr~tr~rc);
print ftosa(100*vol,"vol = %lf",3,2);
print ftosa(results,"%lf",3,2);

p = 2;
let sgn = 1 -1;
{A,D} = regFactorModel(cov,p);
A = A .* sgn';
Omega = eye(p);

print "A = "; print ftosa(100*A,"%lf",3,5);
print "D = "; print ftosa(100*D,"%lf",3,5);

{vol,y,mr,tr,rc} = Compute_Risk_Factor_Contribution(x,cov,A,Omega,D,1);
results = 100*(y~mr~tr~rc);
print ftosa(100*vol,"vol = %lf",3,2);
print ftosa(results,"%lf",3,2);

{x,vol,y,mr,tr,rc} = Compute_RFD_Portfolio(0,cov,A,Omega,D,1);
results = 100*(y~mr~tr~rc);
print ftosa(100*vol,"vol = %lf",3,2);
print ftosa(results,"%lf",3,2);

{vol,y,mr,tr,rc} = Compute_Risk_Factor_Contribution(x,cov,A,Omega,D,0);
results = 100*(y~mr~tr~rc);
print ftosa(100*vol,"vol = %lf",3,2);
print ftosa(results,"%lf",3,2);

/*

print "------------ ERC (Fectors) --------------------";
print (ftosa(100*vol,"vol = %lf",5,2) $+ ftosa(100*sumc(tr)," / tr = %lf",5,2));
print (ftosa(100*f,"%lf",3,1) $~ ftosa(mr~tr,"%lf",3,3) $~ ftosa((tr/vol)*100,"%lf",3,1));

*/